#!/use/bin/env python
# -*- coding: utf-8 -*-
"""Static website generator inspired by jekyll based on flask."""

# Based on the tutorial on https://nicolas.perriault.net/code/2012/dead-easy-\
# yet-powerful-static-website-generator-with-flask/.

import os
import argparse
import shutil
import locale
from datetime import date, datetime
from pkg_resources import resource_filename

from bs4 import BeautifulSoup

from flask import Flask, Blueprint, render_template
from flask import current_app, url_for
from flask import Markup, escape
from flask_flatpages import FlatPages, Page
from flask_frozen import Freezer

__version__ = '0.4.0'

DEBUG = True
FLATPAGES_AUTO_RELOAD = DEBUG
FLATPAGES_EXTENSION = ['.html', '.md']

# http://pythonhosted.org/Markdown/extensions/#officially-supported-extensions
FLATPAGES_MARKDOWN_EXTENSIONS = [
    'codehilite',
    'headerid',
    'fenced_code',
    'footnotes',
    'tables',
    'abbr',
    'wikilinks',
    'toc',
]

# https://docs.python.org/3/library/locale.html#locale.setlocale
locale.setlocale(locale.LC_ALL, '')

flekky = Blueprint('flekky', __name__)


def _(s):
    return s


def shift_headings(html, offset):
    if offset == 0:
        return html

    soup = BeautifulSoup(html, 'html.parser')

    order = range(1, 6)
    if offset > 0:
        order = reversed(order)

    for i in order:
        j = min(max(i + offset, 1), 6)
        for tag in soup.find_all('h%i' % i):
            tag.name = 'h%i' % j

    return soup.prettify()


def page_fix_outline(self, base_heading_level):
    return shift_headings(self.html, base_heading_level - 1)


Page.fix_outline = page_fix_outline


class FlekkyPages(FlatPages):
    """Flat Pages with some extra features for Jekyll compatibility."""

    def _is_included(self, page):
        if not page:
            return False

        include_unpublished = self.app.config.get('FLEKKY_UNPUBLISHED', False)
        if not include_unpublished and not page.meta.get('published', True):
            return False

        include_future = self.app.config.get('FLEKKY_FUTURE', False)
        is_future = 'date' in page.meta and page.meta['date'] > date.today()
        if not include_future and is_future:
            return False

        return True

    def get(self, path, default=None):
        page = super(FlekkyPages, self).get(path)

        if self._is_included(page):
            return page
        else:
            return default

    def __iter__(self):
        for page in super(FlekkyPages, self).__iter__():
            if self._is_included(page) and page.path != 'index':
                yield page

    def by_key(self, key, value, default=None, is_list=False):
        if is_list:
            return (p for p in self if value in p.meta.get(key, []))
        else:
            return (p for p in self if value == p.meta.get(key, default))

    def values(self, key, is_list=False):
        if is_list:
            values = set()
            for page in self:
                values.update(set(page.meta.get(key, [])))
            return values
        else:
            return set([p.meta[key] for p in self if key in p.meta])


pages = FlekkyPages()


# filters
@flekky.app_template_filter('datetime')
def filter_datetime(dt, format="%c"):
    """Convert datetime object to HTML5 markup representing full datetime."""
    return Markup('<time datetime="%s">%s</time>' % (dt, dt.strftime(format)))


@flekky.app_template_filter('date')
def filter_date(dt, format="%x"):
    """Convert datetime object to HTML5 markup representing date only."""
    if isinstance(dt, datetime):
        dt = dt.date()
    return Markup('<time datetime="%s">%s</time>' % (dt, dt.strftime(format)))


@flekky.app_template_filter('time')
def filter_time(dt, format="%X"):
    """Convert datetime object to HTML5 markup representing time only."""
    return Markup('<time datetime="%s">%s</time>' % (dt.time(),
                                                     dt.strftime(format)))


@flekky.app_template_filter('link_page')
def filter_link_page(page):
    """Convert page object to an HTML link to that page."""
    if page is not None:
        href = url_for('flekky.page_route', path=page.path)
        text = page.meta['title']
        return Markup('<a href="%s">%s</a>' % (href, escape(text)))


def _site(_pages):
    """Construct site wide variables.

    ... as opposed to page specific variables.
    """
    site = {
        'title': 'Flekky',
        'time': datetime.now(),
        'pages': _pages,
        'config': current_app.config,
    }

    index = _pages.get('index')
    if hasattr(index, 'meta'):
        site.update(index.meta)

    return site


@flekky.route('/', defaults={'path': 'index'})
@flekky.route('/<path:path>/')
def page_route(path):
    page = pages.get_or_404(path)
    template = 'layout/%s.html' % page.meta.get('layout', 'default')
    return render_template(template, page=page, site=_site(pages))


def create_app(source, settings=None):
    """App factory.

    Args:
        source (str): source directory
        settings (dict): will be loaded after all other configuration is done
    """
    app = Flask(__name__,
                template_folder=os.path.join(source, 'templates'),
                static_folder=os.path.join(source, 'static'))

    app.config.from_object(__name__)
    app.config['FLATPAGES_ROOT'] = os.path.join(source, 'pages')
    app.config.from_object(settings)

    app.register_blueprint(flekky)
    pages.init_app(app)

    return app


def create_freezer(*args, **kwargs):
    """Freezer factory.

    Any arguments will be forwarded to the underlying app factory.
    """
    def urls():
        yield '.page_route', {'path': '/'}
        for page in pages:
            yield '.page_route', {'path': page.path}

    freezer = Freezer(create_app(*args, **kwargs))
    freezer.register_generator(urls)
    return freezer


def parse_args(argv=None):
    """Parse command line arguments.

    Args:
        argv (list): List of command line parameters.
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '--version', '-V', action='version', version=__version__)
    parser.add_argument(
        '--source', '-s', default='_source',
        help=_('directory where Flekky will read files (default: _source)'))
    parser.add_argument(
        '--future', action='store_true', dest='FLEKKY_FUTURE',
        help=_('include pages with dates in the future (default: false)'))
    parser.add_argument(
        '--unpublished', action='store_true', dest='FLEKKY_UNPUBLISHED',
        help=_('include unpublished pages (default: false)'))
    subparsers = parser.add_subparsers(title=_('commands'))

    parser_build = subparsers.add_parser(
        'build', help=_('generate static sites'))
    parser_build.add_argument(
        '--destination', '-d', default=None,
        help=_('directory where Flekky will write files '
               '(default: <source>_build)'))
    parser_build.set_defaults(cmd='build')

    parser_serve = subparsers.add_parser(
        'serve', help=_('run a test server for development'))
    parser_serve.add_argument('--port', '-p', type=int, default=8000)
    parser_serve.set_defaults(cmd='serve')

    parser_build = subparsers.add_parser(
        'init', help=_('bootstrap a new project'))
    parser_build.set_defaults(cmd='init')

    return parser.parse_args(argv)


def rlink(src, dest):
    if os.path.isdir(src):
        if not os.path.exists(dest):
            os.mkdir(dest)
        for filename in os.listdir(src):
            rlink(os.path.join(src, filename), os.path.join(dest, filename))
    elif os.path.lexists(dest):
        if (not os.path.exists(dest) or
                os.path.getmtime(dest) < os.path.getmtime(src)):
            os.unlink(dest)
            os.link(src, dest)
    else:
        os.link(src, dest)


def main():  # pragma: no cover
    args = parse_args()
    source = os.path.abspath(args.source)

    if args.cmd == 'build':
        if args.destination is None:
            args.destination = '%s_build' % args.source
        destination = os.path.abspath(args.destination)
        args.FREEZER_DESTINATION = destination
        freezer = create_freezer(source, args)
        freezer.freeze()

        # copy all additional files
        for filename in os.listdir(source):
            if (filename not in ['pages', 'static', 'templates'] and
                    not filename.startswith('_') and
                    not filename.startswith('.')):
                srcpath = os.path.join(source, filename)
                dstpath = os.path.join(destination, filename)
                rlink(srcpath, dstpath)
    elif args.cmd == 'serve':
        app = create_app(source, args)
        app.run(port=args.port)
    elif args.cmd == 'init':
        init = resource_filename(__name__, 'init')
        shutil.copytree(init, source)
        print(_('Created new project in %s.') % source)
    else:
        raise ValueError('invalid command: %s' % args.cmd)


if __name__ == '__main__':  # pragma: no cover
    main()

# vim: set ts=4 sw=4 sts=4 et:



import unittest

from flekky.flekky import shift_headings


def clean(html):
    return html.replace(' ', '').replace('\n', '')


class TestShiftHeadings(unittest.TestCase):
    def test_no_headings(self):
        actual = clean(shift_headings('<a>test</a>', 1))
        self.assertEqual(actual, '<a>test</a>')

    def test_h1_shift_1(self):
        actual = clean(shift_headings('<h1>test</h1>', 1))
        self.assertEqual(actual, '<h2>test</h2>')

    def test_h1_shift_3(self):
        actual = clean(shift_headings('<h1>test</h1>', 3))
        self.assertEqual(actual, '<h4>test</h4>')

    def test_h1_shift_7(self):
        actual = clean(shift_headings('<h1>test</h1>', 7))
        self.assertEqual(actual, '<h6>test</h6>')

    def test_h1_shift_0(self):
        actual = clean(shift_headings('<h1>test</h1>', 0))
        self.assertEqual(actual, '<h1>test</h1>')

    def test_h1_unshift_1(self):
        actual = clean(shift_headings('<h1>test</h1>', -1))
        self.assertEqual(actual, '<h1>test</h1>')

    def test_mixed_shift_1(self):
        actual = clean(shift_headings('<h1>test</h1><h3>foo</h3>'
            '<div><h4>baz</h4></div>', 1))
        self.assertEqual(actual, '<h2>test</h2><h4>foo</h4>'
            '<div><h5>baz</h5></div>')

    def test_mixed_unshift_2(self):
        actual = clean(shift_headings('<h1>test</h1><h3>foo</h3>'
            '<div><h4>baz</h4></div>', -2))
        self.assertEqual(actual, '<h1>test</h1><h1>foo</h1>'
            '<div><h2>baz</h2></div>')



#!/usr/bin/env python

import os
import re
from setuptools import setup, find_packages

DIRNAME = os.path.abspath(os.path.dirname(__file__))
rel = lambda *parts: os.path.abspath(os.path.join(DIRNAME, *parts))

README = open(rel('README.rst')).read()
FLEKKY = open(rel('flekky', 'flekky.py')).read()
VERSION = re.search("__version__ = '([^']+)'", FLEKKY).group(1)


setup(
    name='flekky',
    version=VERSION,
    description="Static website generator inspired by jekyll based on flask.",
    long_description=README,
    author='Tobias Bengfort',
    author_email='tobias.bengfort@gmx.net',
    url='https://github.com/xi/flekky',
    install_requires=[
        'Flask>=0.10.1',
        'Flask-FlatPages>=0.6',
        'Frozen-Flask>=0.11',
        'beautifulsoup4>=4.4.0',
        'argparse>=1.2.1',
    ],
    test_suite='test',
    platforms='any',
    packages=find_packages(),
    package_data={'flekky': [
        'init/pages/index.md',
        'init/static/css/style.css',
        'init/templates/base.html',
        'init/templates/layout/default.html',
    ]},
    license='GPLv3+',
    entry_points={'console_scripts': 'flekky=flekky.flekky:main'},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'License :: OSI Approved :: GNU General Public License v3 or later '
            '(GPLv3+)',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Filters',
        'Topic :: Text Processing :: Markup',
        'Topic :: Text Processing :: Markup :: HTML',
    ])



import sys
import os
from random import randint
from shutil import rmtree
from time import sleep
import locale

from datetime import datetime
from flask import Markup
from werkzeug.exceptions import NotFound

import unittest

root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, root)

locale.setlocale(locale.LC_ALL, 'C')

from flekky import flekky  # noqa

if sys.version_info[0] < 3:
    from StringIO import StringIO
    _str = unicode  # noqa
else:
    from io import StringIO
    _str = str


ENVIRON = {
    'SERVER_NAME': 'localhost',
    'wsgi.url_scheme': 'http',
    'SERVER_PORT': '80',
    'REQUEST_METHOD': 'GET',
    'PATH_INFO': '/',
}


class TestCase(unittest.TestCase):
    def setUp(self):
        source = os.path.join(root, '_example')
        self.app = flekky.create_app(source)
        self.pages = flekky.FlekkyPages(self.app)


class TestFlekkyPages(TestCase):
    def test_get(self):
        self.assertTrue(self.pages.get('test') is not None)
        self.assertTrue(self.pages.get('draft') is None)
        self.assertTrue(self.pages.get('future') is None)
        self.assertTrue(self.pages.get('nonexistent') is None)

    def test_iter(self):
        actual = set([p.path for p in self.pages])
        expected = set(['lorem ipsum', 'test', 'tag/test', 'tag/example',
                        'category/greeting'])
        self.assertSetEqual(actual, expected)

    def test_by_tag(self):
        by_tag = self.pages.by_key('tags', 'test', is_list=True)
        paths = set([p.path for p in by_tag])
        self.assertSetEqual(paths, set(['test']))

    def test_by_category(self):
        by_category = self.pages.by_key('category', 'greeting')
        paths = set([p.path for p in by_category])
        self.assertSetEqual(paths, set(['test']))

    def test_tags(self):
        actual = self.pages.values('tags', is_list=True)
        expected = set(['test', 'example'])
        self.assertSetEqual(actual, expected)

    def test_categories(self):
        self.assertSetEqual(self.pages.values('category'), set(['greeting']))


class TestFilters(TestCase):
    dt = datetime(2012, 3, 20, 20)

    def test_datetime(self):
        expected = Markup('<time datetime="2012-03-20 20:00:00">'
            'Tue Mar 20 20:00:00 2012</time>')
        actual = flekky.filter_datetime(self.dt)
        self.assertEqual(expected, actual)

    def test_date(self):
        expected = Markup('<time datetime="2012-03-20">03/20/12</time>')
        actual = flekky.filter_date(self.dt)
        self.assertEqual(expected, actual)

    def test_time(self):
        expected = Markup('<time datetime="20:00:00">20:00:00</time>')
        actual = flekky.filter_time(self.dt)
        self.assertEqual(expected, actual)

    def test_link_page(self):
        expected = Markup('<a href="/test/">Hello World</a>')
        page = self.pages.get('test')
        with self.app.request_context(ENVIRON):
            actual = flekky.filter_link_page(page)
        self.assertEqual(expected, actual)

        with self.app.request_context(ENVIRON):
            self.assertIsNone(flekky.filter_link_page(None))


class TestRoutes(TestCase):
    def test_page(self):
        with self.app.request_context(ENVIRON):
            self.assertTrue(isinstance(flekky.page_route('test'), _str))
            self.assertRaises(NotFound, flekky.page_route, 'nonexistent')


class TestFreeze(unittest.TestCase):
    def test_freeze(self):
        source = os.path.join(root, '_example')
        self.freezer = flekky.create_freezer(source)
        expected = set(['/static/css/style.css', '/test/', '/lorem ipsum/',
                        '/tag/test/', '/tag/example/', '/category/greeting/',
                        '/'])
        actual = set(self.freezer.all_urls())
        self.assertSetEqual(actual, expected)


class TestArgs(unittest.TestCase):
    def setUp(self):
        self._stdout = sys.stdout
        self.out = StringIO()
        sys.stdout = self.out

        self._stderr = sys.stderr
        self.err = StringIO()
        sys.stderr = self.err

    def tearDown(self):
        sys.stdout.close()
        sys.stdout = self._stdout

        sys.stderr.close()
        sys.stderr = self._stderr

    def get_out(self):
        return self.out.getvalue().strip()

    def get_err(self):
        return self.err.getvalue().strip()

    def test_serve_default(self):
        args = flekky.parse_args(['serve'])
        self.assertEqual(args.cmd, 'serve')
        self.assertEqual(args.source, '_source')
        self.assertFalse(args.FLEKKY_FUTURE)
        self.assertFalse(args.FLEKKY_UNPUBLISHED)
        self.assertEqual(args.port, 8000)

    def test_build_default(self):
        args = flekky.parse_args(['build'])
        self.assertEqual(args.cmd, 'build')
        self.assertEqual(args.source, '_source')
        self.assertFalse(args.FLEKKY_FUTURE)
        self.assertFalse(args.FLEKKY_UNPUBLISHED)
        self.assertIsNone(args.destination)

    def test_invalid_cmd(self):
        self.assertRaises(SystemExit, flekky.parse_args, ['invalid'])
        self.assertTrue(self.get_err().startswith('usage'))

    def test_help(self):
        self.assertRaises(SystemExit, flekky.parse_args, ['-h'])
        self.assertTrue(self.get_out().startswith('usage'))


class TestRLink(unittest.TestCase):
    def setUp(self):
        self.dirname = os.path.abspath('.tmp_%i' % randint(1000, 10000))
        os.mkdir(self.dirname)

    def tearDown(self):
        rmtree(self.dirname)

    def touch(self, path):
        with open(path, 'a'):
            pass

    def test_file(self):
        src = os.path.join(self.dirname, 'file')
        dest = os.path.join(self.dirname, 'link')
        self.touch(src)
        flekky.rlink(src, dest)
        self.assertTrue(os.path.samefile(src, dest))

    def test_file_exists_newer(self):
        src1 = os.path.join(self.dirname, 'file1')
        src2 = os.path.join(self.dirname, 'file2')
        dest = os.path.join(self.dirname, 'link')
        self.touch(src1)
        self.touch(src2)
        flekky.rlink(src1, dest)
        flekky.rlink(src2, dest)
        self.assertFalse(os.path.samefile(src1, src2))
        self.assertTrue(os.path.samefile(src1, dest))
        self.assertFalse(os.path.samefile(src2, dest))

    def test_file_exists_older(self):
        src1 = os.path.join(self.dirname, 'file1')
        src2 = os.path.join(self.dirname, 'file2')
        dest = os.path.join(self.dirname, 'link')
        self.touch(src1)
        flekky.rlink(src1, dest)
        sleep(0.1)
        self.touch(src2)
        flekky.rlink(src2, dest)
        self.assertFalse(os.path.samefile(src1, src2))
        self.assertFalse(os.path.samefile(src1, dest))
        self.assertTrue(os.path.samefile(src2, dest))

    def test_dir(self):
        src = os.path.join(self.dirname, 'src')
        dest = os.path.join(self.dirname, 'dest')
        os.mkdir(src)
        self.touch(os.path.join(src, 'file'))

        flekky.rlink(src, dest)
        self.assertTrue(os.path.samefile(
            os.path.join(src, 'file'),
            os.path.join(dest, 'file')))

    def test_dir_dir(self):
        src = os.path.join(self.dirname, 'src')
        dest = os.path.join(self.dirname, 'dest')
        os.mkdir(src)
        os.mkdir(os.path.join(src, 'dir'))
        self.touch(os.path.join(src, 'dir', 'file'))

        flekky.rlink(src, dest)
        self.assertTrue(os.path.samefile(
            os.path.join(src, 'dir', 'file'),
            os.path.join(dest, 'dir', 'file')))

    def test_empty_dir(self):
        src = os.path.join(self.dirname, 'src')
        dest = os.path.join(self.dirname, 'dest')
        os.mkdir(src)
        os.mkdir(os.path.join(src, 'dir'))

        flekky.rlink(src, dest)
        self.assertTrue(os.path.isdir(os.path.join(src, 'dir')))

    def test_dir_exists_file(self):
        src = os.path.join(self.dirname, 'src')
        dest = os.path.join(self.dirname, 'dest')
        os.mkdir(src)
        os.mkdir(dest)
        os.mkdir(os.path.join(src, 'dir'))
        self.touch(os.path.join(dest, 'dir'))

        flekky.rlink(src, dest)
        self.assertTrue(os.path.isdir(os.path.join(src, 'dir')))


if __name__ == '__main__':
    unittest.main()

# vim: set ts=4 sw=4 sts=4 et:
